/*
 * Copyright (C) 2003 Borja Sanchez Zamorano <borsanza@terra.es>
 */

#include <sys/socket.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/ioctl.h>
#include <sys/un.h>
#include <sys/stat.h>

typedef struct ListFD {
  int fd;
  struct ListFD *next;
} LISTFD;

struct ListFD *firstFD;

void add_to_list(int fd) {
  struct ListFD *newFD;

  newFD = (LISTFD *)malloc(sizeof(LISTFD));
  newFD->fd = fd;
  newFD->next = firstFD;
  firstFD = newFD;
}

void delete_node(int fd) {
  struct ListFD **auxFD;
  auxFD = &firstFD;
  while ((*auxFD != NULL) && ((*auxFD)->fd != fd)) {
    auxFD = &((*auxFD)->next);
  }

  if (*auxFD != NULL) {
    struct ListFD *deleteFD;
    deleteFD = *auxFD;
    *auxFD = (*auxFD)->next;
    free(deleteFD);
  }
}

int countFD() {
  struct ListFD *auxFD;
  auxFD = firstFD;
  int cont=0;
  while (auxFD != NULL) {
    ++cont;
    auxFD = auxFD->next;
  }
  cont--;
  return cont;
}

void *listenUnix(void *p) {
  int len;
  int socket_fd, client_fd;
  struct sockaddr_un server, client_addr;
  size_t server_addr_len, client_addr_len;
  int listen_val, fd_value, chrs_read;
  fd_set readset, tmpset;
  char *buff;
  double x_val, sqrt_xval;
  int ret_val, result;
  struct ListFD *auxFD;

  mode_t st_mode;
  st_mode = S_IFDIR | S_IRWXU | S_IRWXG | S_IRWXO;
  mkdir("/tmp/.Cartoon-unix", st_mode);

  firstFD = NULL;

  /* Allocate memory space for buffer */

  buff = (char *)malloc(256);
  if (!buff) {
     perror("Memory allocation failure");
     exit(EXIT_FAILURE);
  }

  /* Create the socket to be used by the server */

  socket_fd = socket(PF_UNIX, SOCK_STREAM, 0);
  if (socket_fd == -1) {
      perror("Cannot create socket");
      exit(EXIT_FAILURE);
  }

  /* If the socket was successfully created, name  the socket. */

  server.sun_family = PF_UNIX;
  strcpy(server.sun_path, "/tmp/.Cartoon-unix/C0" );
  ret_val = bind(socket_fd, (struct sockaddr *)&server, sizeof(server));
  if (ret_val == -1) {
     perror("Cannot name socket (/tmp/.Cartoon-unix/C0)");
  }

  /* Set the server and socket queues */

  listen_val = listen(socket_fd, 5);
  if (listen_val == -1) {
    perror("Could not create the socket queue");
    exit(EXIT_FAILURE);
  }

  /* Initialize the socket descriptor set, add the server  socket descriptor *
   * to the set, add the socket descriptor to the list.                      */

  FD_ZERO(&readset);
  FD_SET(socket_fd, &readset);
  add_to_list(socket_fd);

  /* Wait for connecting clients and client request */

  while (1) {

     tmpset = readset;
     auxFD = firstFD;

     /* Block until a socket descriptor in the set shows activity. */

     result = select(FD_SETSIZE, &tmpset, NULL, NULL, NULL);
     if (result < 1) {
       fprintf(stderr, "Critical server error occurred...exiting. \n");
       perror("netserv5");
       exit(EXIT_FAILURE);
     }
     else {    /* detected activity on a socket descriptor */
       while (auxFD != NULL) {
         if (FD_ISSET(auxFD->fd, &tmpset)) {
           if (auxFD->fd == socket_fd) {    /* New client trying to establish connection */
             client_fd = accept(socket_fd, (struct sockaddr *)&client_addr, &client_addr_len);
             FD_SET(client_fd, &readset);  /* add new client descriptor to set */
             add_to_list(client_fd);
             printf("%d: Conectado (conectados %d)\n", client_fd, countFD());
           }
           else {   /* if activity occurs on a connected client descriptor */
             ioctl(auxFD->fd, FIONREAD, &chrs_read);
             switch (chrs_read) {
               case 0:                 /* If client is closing connection with server */
                 printf("%d: Desconectado (conectados %d)\n", auxFD->fd, countFD()-1);
                 close(auxFD->fd);
                 FD_CLR(auxFD->fd, &readset);  /* remove client descriptor from set */
                 delete_node(auxFD->fd);  /* remove client fd from list */
                 break;

               default:    /* Handle client request as normal */
                 len = read(auxFD->fd, buff, 256);
                 buff[len] = '\0';
                 printf("%d: Recibido %d bytes: %s\n", auxFD->fd, len, buff);
                 break;
             }
           }
         }
         auxFD = auxFD->next;
       }
     }
  }
  free(buff);
  free(firstFD);

  exit(EXIT_SUCCESS);
}
