/*
 * Copyright (C) 2003 Borja Sanchez Zamorano <borsanza@terra.es>
 */

#include <math.h>
#include <stdlib.h>

#include "dock.h"

#define MINSIZE 32
#define MAXSIZE 144
#define SEPARATION 128  // MINSIZE * 3
//#define DEBUG_EVENTS

/*****************************************************************************/

Dock::Dock() {
  int i;
#ifdef DEBUG_EVENTS
  printf("Dock::Dock()\n");
#endif
  FirstIcon = NULL;
  LastIcon = NULL;

  dockText[0] = pngBind("./themes/aqua/dock/dock0.png", PNG_NOMIPMAPS, PNG_ALPHA, &dockInfo[0], GL_CLAMP, GL_NEAREST, GL_NEAREST);
  dockText[1] = pngBind("./themes/aqua/dock/dock1.png", PNG_NOMIPMAPS, PNG_ALPHA, &dockInfo[1], GL_CLAMP, GL_NEAREST, GL_NEAREST);
  dockText[2] = pngBind("./themes/aqua/dock/dock2.png", PNG_NOMIPMAPS, PNG_ALPHA, &dockInfo[2], GL_CLAMP, GL_NEAREST, GL_NEAREST);

  for (i=1; i<=6; i++)
    addIcon(i);
}

/*****************************************************************************/

void Dock::refreshSize() {
#ifdef DEBUG_EVENTS
  printf("Dock::refreshSize()\n");
#endif
  struct ListIcons *auxIcon;
  auxIcon = FirstIcon;
  int aux;
  while (auxIcon != NULL) {
    aux = abs(mouseX - auxIcon->posX);
    if (aux < (SEPARATION)) {
      auxIcon->width = MINSIZE + ((MAXSIZE-MINSIZE)*sin((M_PI_2 * (SEPARATION - aux))/SEPARATION));
    } else
      auxIcon->width = MINSIZE;
    auxIcon = auxIcon->next;
  }

  int min = width;
  struct ListIcons *minIcon = FirstIcon;

  // Decidir cual es el icono mas cercano al raton
  auxIcon = FirstIcon;
  while (auxIcon != NULL) {
    if (abs(mouseX - auxIcon->posX)< min) {
      min = abs(mouseX - auxIcon->posX);
      minIcon = auxIcon;
    }
    auxIcon = auxIcon->next;
  }

  minIcon->newX = minIcon->posX;
  double por = (double)(mouseX - minIcon->posX) / MINSIZE;

  if ((mouseX > minIcon->posX) && (minIcon->next != NULL)) {
    float size = (minIcon->next->width + minIcon->width)/2;
    size = MINSIZE - size;
    size = size * por;
    minIcon->newX = minIcon->posX + size;
  }

  if ((mouseX < minIcon->posX) && (minIcon->prev != NULL)) {
    float size = (minIcon->prev->width + minIcon->width)/2;
    size = MINSIZE - size;
    size = size * por;
    minIcon->newX = minIcon->posX + size;
  }

  double size, size2 = minIcon->width;
  double pos, pos2 = minIcon->newX;
  pos = pos2;
  size = size2;

  // Colocar los iconos siguientes al seleccionado
  auxIcon = minIcon->next;
  while (auxIcon != NULL) {
    auxIcon->newX = pos + (size + auxIcon->width) / 2;
    pos = auxIcon->newX;
    size = auxIcon->width;
    auxIcon = auxIcon->next;
  }

  // Colocar los iconos anteriores al seleccionado
  pos = pos2;
  size = size2;
  auxIcon = minIcon->prev;
  while (auxIcon != NULL) {
    auxIcon->newX = pos - (size + auxIcon->width) / 2;
    pos = auxIcon->newX;
    size = auxIcon->width;
    auxIcon = auxIcon->prev;
  }
}

/*****************************************************************************/

void Dock::refreshListInverse() {
#ifdef DEBUG_EVENTS
  printf("Dock::refreshListInverse()\n");
#endif
  struct ListIcons *auxList, *auxPrev=NULL;
  auxList=FirstIcon;
  while (auxList != NULL) {
    auxList->prev = auxPrev;
    auxPrev = auxList;
    auxList = auxList->next;
  }
  LastIcon = auxPrev;
}

/*****************************************************************************/

void Dock::refreshList() {
#ifdef DEBUG_EVENTS
  printf("Dock::refreshList()\n");
#endif
  int aux = 0;
  struct ListIcons *auxIcon;
  auxIcon = FirstIcon;
  while (auxIcon != NULL) {
    auxIcon->posX = aux;
    aux += MINSIZE;
    auxIcon = auxIcon->next;
  }

  aux = (width-aux+MINSIZE)>>1;

  auxIcon = FirstIcon;
  while (auxIcon != NULL) {
    auxIcon->posX += aux;
    auxIcon = auxIcon->next;
  }
  refreshSize();
}

/*****************************************************************************/

void Dock::addIcon(int pos) {
#ifdef DEBUG_EVENTS
  printf("Dock::addIcon()\n");
#endif
  struct ListIcons *newIcon;
  newIcon = (ListIcons *) malloc (sizeof(ListIcons));
  newIcon->next = FirstIcon;
  FirstIcon = newIcon;
	switch (pos) {
		case 6:
			newIcon->imgText = pngBind("./themes/aqua/icons/128x128/mac.png", PNG_NOMIPMAPS, PNG_ALPHA, &newIcon->imgInfo, GL_CLAMP, GL_LINEAR, GL_LINEAR);
      newIcon->imgMiniText = pngBind("./themes/aqua/icons/64x64/mac.png", PNG_NOMIPMAPS, PNG_ALPHA, &newIcon->imgMiniInfo, GL_CLAMP, GL_LINEAR,GL_LINEAR);
			break;
		case 5:
			newIcon->imgText = pngBind("./themes/aqua/icons/128x128/safari.png", PNG_NOMIPMAPS, PNG_ALPHA, &newIcon->imgInfo, GL_CLAMP, GL_LINEAR, GL_LINEAR);
			newIcon->imgMiniText = pngBind("./themes/aqua/icons/64x64/safari.png", PNG_NOMIPMAPS, PNG_ALPHA, &newIcon->imgMiniInfo, GL_CLAMP, GL_LINEAR,GL_LINEAR);
			break;
		case 4:
			newIcon->imgText = pngBind("./themes/aqua/icons/128x128/mail.png", PNG_NOMIPMAPS, PNG_ALPHA, &newIcon->imgInfo, GL_CLAMP, GL_LINEAR, GL_LINEAR);
      newIcon->imgMiniText = pngBind("./themes/aqua/icons/64x64/mail.png", PNG_NOMIPMAPS, PNG_ALPHA, &newIcon->imgMiniInfo, GL_CLAMP, GL_LINEAR,GL_LINEAR);
			break;
		case 3:
			newIcon->imgText = pngBind("./themes/aqua/icons/128x128/dvd_unmount.png", PNG_NOMIPMAPS, PNG_ALPHA, &newIcon->imgInfo, GL_CLAMP, GL_LINEAR, GL_LINEAR);
      newIcon->imgMiniText = pngBind("./themes/aqua/icons/64x64/dvd_unmount.png", PNG_NOMIPMAPS, PNG_ALPHA, &newIcon->imgMiniInfo, GL_CLAMP, GL_LINEAR,GL_LINEAR);
			break;
		case 2:
			newIcon->imgText = pngBind("./themes/aqua/icons/128x128/itunes.png", PNG_NOMIPMAPS, PNG_ALPHA, &newIcon->imgInfo, GL_CLAMP, GL_LINEAR, GL_LINEAR);
      newIcon->imgMiniText = pngBind("./themes/aqua/icons/64x64/itunes.png", PNG_NOMIPMAPS, PNG_ALPHA, &newIcon->imgMiniInfo, GL_CLAMP, GL_LINEAR,GL_LINEAR);
			break;
		case 1:
			newIcon->imgText = pngBind("./themes/aqua/icons/128x128/03_27.png", PNG_NOMIPMAPS, PNG_ALPHA, &newIcon->imgInfo, GL_CLAMP, GL_LINEAR, GL_LINEAR);
			newIcon->imgMiniText = pngBind("./themes/aqua/icons/64x64/03_27.png", PNG_NOMIPMAPS, PNG_ALPHA, &newIcon->imgMiniInfo, GL_CLAMP, GL_LINEAR,GL_LINEAR);
			break;
		default:
			newIcon->imgText = pngBind("./themes/aqua/icons/128x128/penguin.png", PNG_NOMIPMAPS, PNG_ALPHA, &newIcon->imgInfo, GL_CLAMP, GL_LINEAR, GL_LINEAR);
      newIcon->imgMiniText = pngBind("./themes/aqua/icons/64x64/penguin.png", PNG_NOMIPMAPS, PNG_ALPHA, &newIcon->imgMiniInfo, GL_CLAMP, GL_LINEAR,GL_LINEAR);
	}

  refreshListInverse();

  refreshList();
}

/*****************************************************************************/

Dock::~Dock() {
}

/*****************************************************************************/

bool Dock::isMouse() {
  return false;
}

/*****************************************************************************/

void Dock::resizeEvent(int widthP, int heightP) {
#ifdef DEBUG_EVENTS
  printf("Dock::resizeEvent()\n");
#endif
  width = widthP;
  height = heightP;
  refreshList();
}

/*****************************************************************************/

void Dock::mouseMoveEvent(int x, int y) {
  struct ListIcons *auxIcon;
#ifdef DEBUG_EVENTS
  printf("Dock::mouseMoveEvent()\n");
#endif
  mouseX = x;
  mouseY = y;
  static bool activo=false;
  if (mouseY>(height-(MINSIZE+10)))
    activo=true;

  auxIcon = FirstIcon;
  float max=0.0;
  while (auxIcon != NULL) {
    if (auxIcon->width > max) max=auxIcon->width;
    auxIcon = auxIcon->next;
  }

  if ((mouseY<(height-(max+10)))&&(activo))
    activo=false;
  if (!activo)
    mouseX = -10000;
  refreshList();
}

/*****************************************************************************/

void Dock::paintEvent() {
#ifdef DEBUG_EVENTS
  printf("Dock::paintEvent()\n");
#endif
  struct ListIcons *auxIcon;

  glColor3f(1.0f, 1.0f, 1.0f);

  glBindTexture(GL_TEXTURE_2D, dockText[0]);
  glBegin(GL_QUADS);
  glTexCoord2f(0.0f, 0.0f); glVertex2f(FirstIcon->newX - (FirstIcon->width/2) - 5 - dockInfo[0].Width, height - (MINSIZE + 10));
  glTexCoord2f(0.0f, 1.0f); glVertex2f(FirstIcon->newX - (FirstIcon->width/2) - 5 - dockInfo[0].Width, height - (MINSIZE + 10) + dockInfo[1].Height);
  glTexCoord2f(1.0f, 1.0f); glVertex2f(FirstIcon->newX - (FirstIcon->width/2) - 5, height - (MINSIZE + 10) + dockInfo[0].Height);
  glTexCoord2f(1.0f, 0.0f); glVertex2f(FirstIcon->newX - (FirstIcon->width/2) - 5, height - (MINSIZE + 10));
  glEnd();

  glBindTexture(GL_TEXTURE_2D, dockText[2]);
  glBegin(GL_QUADS);
  glTexCoord2f(0.0f, 0.0f); glVertex2f(LastIcon->newX + (LastIcon->width/2) + 5, height - (MINSIZE + 10));
  glTexCoord2f(0.0f, 1.0f); glVertex2f(LastIcon->newX + (LastIcon->width/2) + 5, height - (MINSIZE + 10) + dockInfo[1].Height);
  glTexCoord2f(1.0f, 1.0f); glVertex2f(LastIcon->newX + (LastIcon->width/2) + 5 + dockInfo[2].Width, height - (MINSIZE + 10) + dockInfo[2].Height);
  glTexCoord2f(1.0f, 0.0f); glVertex2f(LastIcon->newX + (LastIcon->width/2) + 5 + dockInfo[2].Width, height - (MINSIZE + 10));
  glEnd();

  glBindTexture(GL_TEXTURE_2D, dockText[1]);
  glBegin(GL_QUADS);
  glTexCoord2f(0.0f, 0.0f); glVertex2f(FirstIcon->newX - (FirstIcon->width/2) - 5, height - (MINSIZE + 10));
  glTexCoord2f(0.0f, 1.0f); glVertex2f(FirstIcon->newX - (FirstIcon->width/2) - 5, height - (MINSIZE + 10) + dockInfo[2].Height);
  glTexCoord2f(1.0f, 1.0f); glVertex2f(LastIcon->newX + (LastIcon->width/2) + 5, height - (MINSIZE + 10) + dockInfo[2].Height);
  glTexCoord2f(1.0f, 0.0f); glVertex2f(LastIcon->newX + (LastIcon->width/2) + 5, height - (MINSIZE + 10));
  glEnd();

  auxIcon = FirstIcon;
  while (auxIcon != NULL) {
    if (auxIcon->width <= 64)
      glBindTexture(GL_TEXTURE_2D, auxIcon->imgMiniText);
    else
      glBindTexture(GL_TEXTURE_2D, auxIcon->imgText);
    glBegin(GL_QUADS);
    glTexCoord2f(0.0f, 0.0f); glVertex2f(auxIcon->newX - (auxIcon->width / 2), height - auxIcon->width-5);
    glTexCoord2f(0.0f, 1.0f); glVertex2f(auxIcon->newX - (auxIcon->width / 2 ), height-5);
    glTexCoord2f(1.0f, 1.0f); glVertex2f(auxIcon->newX + (auxIcon->width / 2), height-5);
    glTexCoord2f(1.0f, 0.0f); glVertex2f(auxIcon->newX + (auxIcon->width / 2), height - auxIcon->width-5);
    glEnd();
    auxIcon = auxIcon->next;
  }
}
